<?php

declare(strict_types=1);

namespace Drupal\commerce_config_actions\Plugin\ConfigAction;

use Drupal\Core\Config\Action\Attribute\ConfigAction;
use Drupal\Core\Config\Action\ConfigActionPluginInterface;
use Drupal\Core\Config\ConfigManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\user\PermissionHandlerInterface;
use Drupal\user\RoleInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * @internal
 *   This API is experimental.
 */
#[ConfigAction(
  id: 'grantExistingPermissions',
  admin_label: new TranslatableMarkup('Add permission to role'),
  entity_types: ['user_role'],
)]
final class GrantExistingPermissions implements ConfigActionPluginInterface, ContainerFactoryPluginInterface {

  /**
   * Constructs a CurrencyImport object.
   *
   * @param \Drupal\Core\Config\ConfigManagerInterface $configManager
   *   The config manager.
   * @param \Drupal\user\PermissionHandlerInterface $permissionHandler
   *   The permission handler.
   */
  public function __construct(
    protected readonly ConfigManagerInterface $configManager,
    protected readonly PermissionHandlerInterface $permissionHandler,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    return new static(
      $container->get('config.manager'),
      $container->get('user.permissions'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function apply(string $configName, mixed $permissions): void {
    if (!is_array($permissions)) {
      $permissions = [$permissions];
    }
    $existing_permissions = $this->permissionHandler->getPermissions();
    $permissions = array_filter($permissions, function ($perm) use ($existing_permissions) {
      return isset($existing_permissions[$perm]);
    });
    $role = $this->configManager->loadConfigEntityByName($configName);
    assert($role instanceof RoleInterface);
    foreach ($permissions as $permission) {
      $role = $role->grantPermission($permission);
    }
    $role->save();
  }

}
